/* GUI_SoundCloudLibrary.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "ui/GUI_SoundcloudLibrary.h"


GUI_SoundCloudLibrary::GUI_SoundCloudLibrary(SoundcloudLibrary* library, QWidget *parent) :
	GUI_AbstractLibrary(library, parent),
	Ui::GUI_SoundcloudLibrary()
{
	setup_parent(this);

	_artist_search = new GUI_SoundcloudArtistSearch(library, this);

	setAcceptDrops(false);

	connect(btn_add, &QPushButton::clicked, this, &GUI_SoundCloudLibrary::btn_add_clicked);

	tb_title->set_rc_menu(ENTRY_PLAY_NEXT | ENTRY_INFO | ENTRY_DELETE | ENTRY_APPEND | ENTRY_REFRESH);
	lv_album->set_rc_menu(ENTRY_PLAY_NEXT | ENTRY_INFO | ENTRY_DELETE | ENTRY_APPEND | ENTRY_REFRESH);
	lv_artist->set_rc_menu(ENTRY_PLAY_NEXT | ENTRY_INFO | ENTRY_DELETE | ENTRY_APPEND | ENTRY_REFRESH);

	btn_info->hide();

	library->load();
}


QComboBox* GUI_SoundCloudLibrary::get_libchooser() const
{
	return combo_lib_chooser;
}


AbstractLibrary::TrackDeletionMode GUI_SoundCloudLibrary::show_delete_dialog(int n_tracks){

	Q_UNUSED(n_tracks)
	return AbstractLibrary::TrackDeletionMode::OnlyLibrary;
}

void GUI_SoundCloudLibrary::btn_add_clicked(){
	_artist_search->show();
}




SoundcloudLibraryContainer::SoundcloudLibraryContainer(QObject *parent) :
	LibraryContainerInterface(parent)
{

}

QString SoundcloudLibraryContainer::get_name() const
{
	return "soundcloud";
}

QString SoundcloudLibraryContainer::get_display_name() const
{
	return tr("Soundcloud");
}

QIcon SoundcloudLibraryContainer::get_icon() const
{
	return QIcon(":/sc_icons/ui/icon.png");
}


QWidget* SoundcloudLibraryContainer::get_ui() const
{
	return static_cast<QWidget*>(ui);
}

QComboBox* SoundcloudLibraryContainer::get_libchooser()
{
	return ui->get_libchooser();
}

void SoundcloudLibraryContainer::init_ui()
{
	SoundcloudLibrary* library = new SoundcloudLibrary(this);
	ui = new GUI_SoundCloudLibrary(library);
}

