/* application.h */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APPLICATION_H
#define APPLICATION_H

#include <QApplication>
#include <QTranslator>

#ifdef WITH_DBUS
	#include "Components/DBus/DBusHandler.h"
#endif

#include "Components/Playlist/PlaylistHandler.h"

#include "Interfaces/LibraryInterface/LibraryPluginHandler.h"
#include "Interfaces/PlayerPlugin/PlayerPluginHandler.h"

#include "GUI/Player/GUI_Player.h"
#include "GUI/Library/GUI_LocalLibrary.h"
#include "GUI/DirectoryWidget/GUI_DirectoryWidget.h"

#include "GUI/PlayerPlugins/PlaylistChooser/GUI_PlaylistChooser.h"
#include "GUI/PlayerPlugins/AudioConverter/GUI_AudioConverter.h"
#include "GUI/PlayerPlugins/Engine/GUI_LevelPainter.h"
#include "GUI/PlayerPlugins/Engine/GUI_Spectrum.h"
#include "GUI/PlayerPlugins/Stream/GUI_Stream.h"
#include "GUI/PlayerPlugins/Stream/GUI_Podcasts.h"
#include "GUI/PlayerPlugins/Equalizer/GUI_Equalizer.h"
#include "GUI/PlayerPlugins/Bookmarks/GUI_Bookmarks.h"
#include "GUI/PlayerPlugins/Speed/GUI_Speed.h"
#include "GUI/PlayerPlugins/Broadcasting/GUI_Broadcast.h"
#include "GUI/PlayerPlugins/Broadcasting/GUI_BroadcastSetup.h"


#include "GUI/FontConfig/GUI_FontConfig.h"
#include "GUI/Notifications/GUI_Notifications.h"
#include "GUI/LastFM/GUI_LastFM.h"
#include "GUI/StartupDialog/GUI_StartupDialog.h"
#include "GUI/LanguageChooser/GUI_LanguageChooser.h"
#include "GUI/StreamRecorder/GUI_StreamRecorder.h"
#include "GUI/RemoteControl/GUI_RemoteControl.h"

#include <QTime>

class Application : public QApplication, private SayonaraClass
{
    Q_OBJECT

public:
	Application(int& argc, char** argv);
    virtual ~Application();

	bool init(QTranslator* translator, const QStringList& files_to_play);


private:
	void check_for_crash();

	QTime*		_timer;
};


#endif // APPLICATION_H


