/* LibraryImporter.h */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef LIBRARYIMPORTER_H
#define LIBRARYIMPORTER_H

#include "threads/ImportCachingThread.h"
#include "threads/ImportCopyThread.h"
#include "Helper/Tagging/Tagging.h"
#include "Database/DatabaseConnector.h"


class LibraryImporter : public QObject, protected SayonaraClass
{
	Q_OBJECT

	SINGLETON_QOBJECT(LibraryImporter)


public:

	enum class ImportStatus : quint8 {
			 Cancelled,
			 Rollback,
			 Caching,
			 NoTracks,
			 Sleeping,
			 Importing,
			 Imported
	};

signals:
	void sig_got_metadata(const MetaDataList&);
	void sig_status_changed(LibraryImporter::ImportStatus);
	void sig_got_library_dirs(const QStringList& library_dirs);
	void sig_progress(int);

	void sig_imported();
	void sig_triggered();


public:
	void import_files(const QStringList& filelist);
	void import_dir(const QString& dirname);
	void accept_import(const QString& target_dir);
	void cancel_import();


private slots:

	void start_copy_thread();
	void copy_thread_finished();

	void caching_thread_done();
	void caching_thread_finished();

	void metadata_changed(const MetaDataList& old_md, const MetaDataList& new_md);

private:
	ImportCachingThread*    _caching_thread;
	ImportCopyThread*       _copy_thread;
	DatabaseConnector*     _db;

	QString                 _import_to;
	QString                 _lib_path;
	QString					_src_dir;
};

#endif // LIBRARYIMPORTER_H
