/* ContextMenu.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "ContextMenu.h"
#include "Helper/Helper.h"

#include <QAction>

ContextMenu::ContextMenu(QWidget* parent) :
    QMenu(parent)
{
	_info_action = new QAction(Helper::get_icon("info"), tr("Info"), this);
	_lyrics_action  = new QAction(Helper::get_icon("lyrics"), tr("Lyrics"), this);
	_edit_action = new QAction(Helper::get_icon("edit"), tr("Edit"), this);
	_remove_action = new QAction(Helper::get_icon("delete"), tr("Remove"), this);
	_delete_action = new QAction(Helper::get_icon("delete"), tr("Delete"), this);
	_play_next_action = new QAction(Helper::get_icon("fwd_orange"), tr("Play next"), this);
	_append_action = new QAction(Helper::get_icon("append"), tr("Append"), this);
	_refresh_action = new QAction(Helper::get_icon("undo"), tr("Refresh"), this);
}


ContextMenu::~ContextMenu() {

    clear_actions();

    delete _info_action;
	delete _lyrics_action;
    delete _edit_action;
    delete _remove_action;
    delete _delete_action;
    delete _play_next_action;
	delete _refresh_action;
}

void ContextMenu::changeEvent(QEvent* e) {
    if (e->type() == QEvent::LanguageChange) {
        _info_action->setText(tr("Info"));
		_lyrics_action->setText(tr("Lyrics"));
        _edit_action->setText(tr("Edit"));
        _remove_action->setText(tr("Remove"));
        _delete_action->setText(tr("Delete"));
        _play_next_action->setText(tr("Play next"));
        _append_action->setText(tr("Append"));
		_refresh_action->setText(tr("Refresh"));

        return;
    }

    QMenu::changeEvent(e);


}


void ContextMenu::clear_actions() {

    QList<QAction*> actions = this->actions();
    if(actions.size() > 0) {
		for(QAction* a : actions){
			this->removeAction(a);
		}
    }

	disconnect(_info_action, &QAction::triggered, this, &ContextMenu::sig_info_clicked);
	disconnect(_lyrics_action, &QAction::triggered, this, &ContextMenu::sig_lyrics_clicked);
	disconnect(_edit_action, &QAction::triggered, this, &ContextMenu::sig_edit_clicked);
	disconnect(_remove_action, &QAction::triggered, this, &ContextMenu::sig_remove_clicked);
	disconnect(_delete_action, &QAction::triggered, this, &ContextMenu::sig_delete_clicked);
	disconnect(_play_next_action, &QAction::triggered, this, &ContextMenu::sig_play_next_clicked);
	disconnect(_refresh_action, &QAction::triggered, this, &ContextMenu::sig_refresh_clicked);

}

void ContextMenu::setup_entries(int entries) {

    clear_actions();

    if(entries & ENTRY_INFO) {
        this->addAction(_info_action);
		connect(_info_action, &QAction::triggered, this, &ContextMenu::sig_info_clicked);
    }

	if(entries & ENTRY_LYRICS) {
		this->addAction(_lyrics_action);
		connect(_lyrics_action, &QAction::triggered, this, &ContextMenu::sig_lyrics_clicked);
	}

    if(entries & ENTRY_EDIT) {
        this->addAction(_edit_action);
		connect(_edit_action, &QAction::triggered, this, &ContextMenu::sig_edit_clicked);
    }

    if(entries & ENTRY_REMOVE) {
        this->addAction(_remove_action);
		connect(_remove_action, &QAction::triggered, this, &ContextMenu::sig_remove_clicked);
    }

    if(entries & ENTRY_DELETE) {
        this->addAction(_delete_action);
		connect(_delete_action, &QAction::triggered, this, &ContextMenu::sig_delete_clicked);
    }

    if(entries & ENTRY_PLAY_NEXT) {
        this->addAction(_play_next_action);
		connect(_play_next_action, &QAction::triggered, this, &ContextMenu::sig_play_next_clicked);
    }

    if(entries & ENTRY_APPEND) {
        this->addAction(_append_action);
		connect(_append_action, &QAction::triggered, this, &ContextMenu::sig_append_clicked);
    }

	if(entries & ENTRY_REFRESH) {
		this->addAction(_refresh_action);
		connect(_refresh_action, &QAction::triggered, this, &ContextMenu::sig_refresh_clicked);
	}
}
