/* MenuTool.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "MenuTool.h"
#include "Helper/Style/Style.h"
#include "Helper/Helper.h"

MenuToolButton::MenuToolButton(QWidget *parent) :
	MenuToolButton::MenuToolButton(false, parent)
{
}

MenuToolButton::MenuToolButton(bool show_all, QWidget *parent) :
	MenuButton(parent)
{
	_menu = new MenuToolMenu(this);

	connect(_menu, &MenuToolMenu::sig_open, this,  &MenuToolButton::sig_open);
	connect(_menu, &MenuToolMenu::sig_new, this, &MenuToolButton::sig_new);
	connect(_menu, &MenuToolMenu::sig_undo, this, &MenuToolButton::sig_undo);
	connect(_menu, &MenuToolMenu::sig_save, this, &MenuToolButton::sig_save);
	connect(_menu, &MenuToolMenu::sig_save_as, this, &MenuToolButton::sig_save_as);
	connect(_menu, &MenuToolMenu::sig_rename, this, &MenuToolButton::sig_rename);
	connect(_menu, &MenuToolMenu::sig_delete, this, &MenuToolButton::sig_delete);

	_menu->show_all(show_all);

	prove_enabled();

}

MenuToolButton::~MenuToolButton()
{

}


void MenuToolButton::register_action(QAction *action)
{
	_menu->register_action(action);
}


void MenuToolButton::show_open(bool b){
	_menu->show_open(b);
	prove_enabled();
}

void MenuToolButton::show_new(bool b){
	_menu->show_new(b);
	prove_enabled();
}

void MenuToolButton::show_undo(bool b){
	_menu->show_undo(b);
	prove_enabled();
}

void MenuToolButton::show_save(bool b){
	_menu->show_save(b);
	prove_enabled();
}

void MenuToolButton::show_save_as(bool b){
	_menu->show_save_as(b);
	prove_enabled();
}

void MenuToolButton::show_rename(bool b){
	_menu->show_rename(b);
	prove_enabled();
}

void MenuToolButton::show_delete(bool b){
	_menu->show_delete(b);
	prove_enabled();
}

void MenuToolButton::show_all(bool b){
	_menu->show_all(b);
	prove_enabled();
}

void MenuToolButton::show_menu(QPoint pos)
{

	MenuButton::show_menu(pos);

	this->setDisabled(true);
	_menu->popup(pos);
	this->setEnabled(true);
}


bool MenuToolButton::prove_enabled()
{
	bool enabled = _menu->has_actions();
	this->setEnabled(enabled);
	return enabled;
}


MenuToolMenu::MenuToolMenu(QWidget *parent) :
	QMenu(parent)
{
	_timer = new QTimer(this);

	_action_open = new QAction(Helper::get_icon("open"), tr("Open"), this);
	_action_new = new QAction(Helper::get_icon("new"), tr("New"), this);
	_action_undo = new QAction(Helper::get_icon("undo"), tr("Undo"), this);
	_action_save = new QAction(Helper::get_icon("save"), tr("Save"), this);
	_action_save_as = new QAction(Helper::get_icon("save_as"), tr("Save as"), this);
	_action_rename = new QAction(Helper::get_icon("edit"), tr("Rename"), this);
	_action_delete = new QAction(Helper::get_icon("delete"), tr("Delete"), this);


	_actions << addSeparator()
			<< _action_new
			<< _action_open
			<< _action_save
			<< _action_save_as
			<< _action_rename
			<< addSeparator()
			<< _action_undo
			<< addSeparator()
			<< _action_delete
			<< addSeparator();

	this->addActions(_actions);


	connect(_action_open, &QAction::triggered, this, &MenuToolMenu::sig_open);
	connect(_action_new, &QAction::triggered, this, &MenuToolMenu::sig_new);
	connect(_action_undo, &QAction::triggered, this, &MenuToolMenu::sig_undo);
	connect(_action_save, &QAction::triggered, this, &MenuToolMenu::sig_save);
	connect(_action_save_as, &QAction::triggered, this, &MenuToolMenu::sig_save_as);
	connect(_action_rename, &QAction::triggered, this, &MenuToolMenu::sig_rename);
	connect(_action_delete, &QAction::triggered, this, &MenuToolMenu::sig_delete);

	REGISTER_LISTENER(Set::Player_Language, language_changed);
}

void MenuToolMenu::language_changed(){
	 _action_new->setText(tr("New"));
	 _action_open->setText(tr("Open"));
	 _action_save->setText(tr("Save"));
	 _action_save_as->setText(tr("Save as"));
	 _action_rename->setText(tr("Rename"));
	 _action_undo->setText(tr("Undo"));
	 _action_delete->setText(tr("Delete"));
}

void MenuToolMenu::register_action(QAction *action){
	_actions << action;

	clear();
	addActions(_actions);
}


void MenuToolMenu::show_open(bool b){
	_action_open->setVisible(b);
}

void MenuToolMenu::show_new(bool b){
	_action_new->setVisible(b);
}

void MenuToolMenu::show_undo(bool b){
	_action_undo->setVisible(b);
}

void MenuToolMenu::show_save(bool b){
	_action_save->setVisible(b);
}

void MenuToolMenu::show_save_as(bool b){
	_action_save_as->setVisible(b);
}

void MenuToolMenu::show_rename(bool b){
	_action_rename->setVisible(b);
}

void MenuToolMenu::show_delete(bool b){
	_action_delete->setVisible(b);
}

bool MenuToolMenu::has_actions() {
	for(QAction* action: _actions){
		if(action->isVisible()){
			return true;
		}
	}

	return false;
}



void MenuToolMenu::show_all(bool b){
	for(QAction* action: _actions){
		action->setVisible(b);
	}
}

void MenuToolMenu::showEvent(QShowEvent* e){
	QMenu::showEvent(e);
	for(QAction* action: _actions){
		action->setDisabled(true);
	}

	QTimer::singleShot(300, this, SLOT(timed_out()));
}

void MenuToolMenu::timed_out(){
	for(QAction* action: _actions){
		action->setDisabled(false);
	}
}

