/* MenuTool.h */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef MENUTOOL_H
#define MENUTOOL_H

#include <QMenu>

#include <QMouseEvent>
#include <QList>
#include <QAction>
#include <QTimer>

#include "MenuButton.h"

class MenuToolMenu :
		public QMenu,
		protected SayonaraClass
{
	Q_OBJECT

signals:
	void sig_new();
	void sig_undo();
	void sig_save();
	void sig_save_as();
	void sig_rename();
	void sig_delete();
	void sig_open();


private:
	QAction*	_action_new=nullptr;
	QAction*	_action_open=nullptr;
	QAction*	_action_undo=nullptr;
	QAction*	_action_save=nullptr;
	QAction*	_action_save_as=nullptr;
	QAction*	_action_rename=nullptr;
	QAction*	_action_delete=nullptr;


	QList<QAction*>		_actions;
	QTimer*				_timer=nullptr;

	void show_action(bool b, QAction* action);


public:
	explicit MenuToolMenu(QWidget *parent=nullptr);
	void register_action(QAction* action);
	bool has_actions();


protected:
	void showEvent(QShowEvent* e) override;


public slots:
	void show_open(bool b);
	void show_new(bool b);
	void show_undo(bool b);
	void show_save(bool b);
	void show_save_as(bool b);
	void show_rename(bool b);
	void show_delete(bool b);
	void show_all(bool b);


private slots:
	void timed_out();
	void language_changed();
};


class MenuToolButton : public MenuButton {

	Q_OBJECT

signals:
	void sig_open();
	void sig_new();
	void sig_undo();
	void sig_save();
	void sig_save_as();
	void sig_rename();
	void sig_delete();

public:
	MenuToolButton(QWidget *parent=nullptr);
	MenuToolButton(bool show_all=false, QWidget *parent=nullptr);
	virtual ~MenuToolButton();

	/**
	 * @brief Use this to add custom actions
	 * @param action
	 */
	void register_action(QAction* action);



private:
	MenuToolMenu* _menu=nullptr;

	void show_menu(QPoint pos) override;
	bool prove_enabled() override;


public slots:
	void show_open(bool b);
	void show_new(bool b);
	void show_undo(bool b);
	void show_save(bool b);
	void show_save_as(bool b);
	void show_rename(bool b);
	void show_delete(bool b);
	void show_all(bool b);
};




#endif // MENUTOOL_H
