/* LibraryGenreView.h */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef LIBRARYGENREVIEW_H
#define LIBRARYGENREVIEW_H

#include "GUI/Helper/Delegates/TreeDelegate.h"
#include "Helper/Tree/Tree.h"

#include <QTreeWidget>
#include <QMap>
#include <QItemSelectionModel>

#include <QDragEnterEvent>
#include <QDragMoveEvent>
#include <QDropEvent>
#include <QDragLeaveEvent>
#include <QShowEvent>

class MetaDataList;
class CustomMimeData;

typedef Tree<QString> GenreNode;


class LibraryGenreView : public QTreeWidget
{
	Q_OBJECT

public:
	LibraryGenreView(QWidget* parent=nullptr);
	~LibraryGenreView();

	QSize sizeHint() const override;
	void reload_genres();
	int get_row_count() const;

private:

	TreeDelegate*			_delegate=nullptr;
	QStringList				_expanded_items;
	bool					_filled;
	GenreNode*				_genres=nullptr;


private:
	void fill_list(const QStringList& genres);
	void init_data(const QStringList& genres);
	void insert_genres(QTreeWidgetItem* parent_item, GenreNode* node);

private slots:
	void metadata_changed(const MetaDataList& v_md_old, const MetaDataList& v_md_new);
	void item_expanded(QTreeWidgetItem* item);
	void item_collapsed(QTreeWidgetItem* item);


protected:
	virtual void dragEnterEvent(QDragEnterEvent* e) override;
	virtual void dragMoveEvent(QDragMoveEvent* e) override;
	virtual void dragLeaveEvent(QDragLeaveEvent* e) override;
	virtual void dropEvent(QDropEvent* e) override;
};

#endif // LIBRARYGENREVIEW_H
