/* PlaylistTabBar.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "PlaylistTabBar.h"



PlaylistTabBar::PlaylistTabBar(QWidget *parent) :
	QTabBar(parent)
{
	_menu = new PlaylistTabMenu(this);
	this->setDrawBase(false);

	connect(_menu, &PlaylistTabMenu::sig_rename_clicked, this, &PlaylistTabBar::rename_pressed);
	connect(_menu, &PlaylistTabMenu::sig_reset_clicked, this, &PlaylistTabBar::reset_pressed);
	connect(_menu, &PlaylistTabMenu::sig_save_clicked, this, &PlaylistTabBar::save_pressed);
	connect(_menu, &PlaylistTabMenu::sig_save_as_clicked, this, &PlaylistTabBar::save_as_pressed);
	connect(_menu, &PlaylistTabMenu::sig_delete_clicked, this, &PlaylistTabBar::delete_pressed);
	connect(_menu, &PlaylistTabMenu::sig_close_clicked, this, &PlaylistTabBar::close_pressed);
	connect(_menu, &PlaylistTabMenu::sig_close_others_clicked, this, &PlaylistTabBar::close_others_pressed);
}


PlaylistTabBar::~PlaylistTabBar(){
	delete _menu;
}


void PlaylistTabBar::save_pressed(){
	emit sig_tab_save(currentIndex());
}

void PlaylistTabBar::save_as_pressed(){
	int cur_idx = currentIndex();
	QString cur_text = tabText(cur_idx);

	QString name = QInputDialog::getText(
				this,
				tr("Save as..."),
				cur_text + ": " + tr("Save as"));

	emit sig_tab_save_as(currentIndex(), name);
}

void PlaylistTabBar::delete_pressed(){
	emit sig_tab_delete(currentIndex());
}

void PlaylistTabBar::close_pressed(){
	emit tabCloseRequested(this->currentIndex());
}

void PlaylistTabBar::reset_pressed(){
	emit sig_tab_reset(currentIndex());
}

void PlaylistTabBar::rename_pressed(){
	int cur_idx = currentIndex();
	QString cur_text = tabText(cur_idx);

	QString name = QInputDialog::getText(
				this,
				tr("Rename"),
				cur_text + ": " + tr("Rename"));

	if(name.compare(cur_text) == 0){
		return;
	}

	emit sig_tab_rename(currentIndex(), name);
}

void PlaylistTabBar::close_others_pressed(){
	int my_tab = currentIndex();
	int i=0;

	while( count() > 1){
		if(i < my_tab){
			tabCloseRequested(0);
		}

		else if(i == my_tab){

		}

		else{
			tabCloseRequested(1);
		}

		i++;
	}
}

void PlaylistTabBar::mousePressEvent(QMouseEvent* e){

	if(e->button() == Qt::RightButton){

		int tab_nr = this->tabAt(e->pos());
		this->setCurrentIndex(tab_nr);
		emit sig_cur_idx_changed(tab_nr);

		_menu->exec(e->globalPos());
	}

	else {
		QTabBar::mousePressEvent(e);
	}
}


void PlaylistTabBar::show_menu_items(bool save_action, bool save_as_action, bool delete_action, bool reset_action){
	_menu->show_menu_items(save_action, save_as_action, delete_action, reset_action);
}

void PlaylistTabBar::setTabsClosable(bool b){
	QTabBar::setTabsClosable(b);
	_menu->show_close(b);
}
