
/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "PlaylistTabMenu.h"


PlaylistTabMenu::PlaylistTabMenu(QWidget* parent) :
	QMenu(parent),
	SayonaraClass()
{

	_action_reset = new QAction(Helper::get_icon("undo"), tr("Reset"), this);
	_action_rename = new QAction(Helper::get_icon("edit"), tr("Rename"), this);
	_action_delete = new QAction(Helper::get_icon("delete"), tr("Delete"), this);
	_action_save = new QAction(Helper::get_icon("save"), tr("Save"), this);
	_action_save_as = new QAction(Helper::get_icon("save_as"), tr("Save as"), this);
	_action_close = new QAction(Helper::get_icon("power_off"), tr("Close"), this);
	_action_close_others = new QAction(Helper::get_icon("power_on"), tr("Close others"), this);


	QList<QAction*> actions;
	actions << _action_reset << this->addSeparator() << _action_rename << _action_save << _action_save_as << _action_delete << this->addSeparator() << _action_close_others << _action_close;

	this->addActions(actions);

	connect(_action_reset, &QAction::triggered, this, &PlaylistTabMenu::sig_reset_clicked);
	connect(_action_rename, &QAction::triggered, this, &PlaylistTabMenu::sig_rename_clicked);
	connect(_action_delete, &QAction::triggered, this, &PlaylistTabMenu::sig_delete_clicked);
	connect(_action_save, &QAction::triggered, this, &PlaylistTabMenu::sig_save_clicked);
	connect(_action_save_as, &QAction::triggered, this, &PlaylistTabMenu::sig_save_as_clicked);
	connect(_action_close, &QAction::triggered, this, &PlaylistTabMenu::sig_close_clicked);
	connect(_action_close_others, &QAction::triggered, this, &PlaylistTabMenu::sig_close_others_clicked);

	REGISTER_LISTENER(Set::Player_Language, language_changed);
}

PlaylistTabMenu::~PlaylistTabMenu(){
	this->clear();
}

void PlaylistTabMenu::language_changed(){
	_action_reset->setText(tr("Reset"));
	_action_rename->setText(tr("Rename"));
	_action_delete->setText(tr("Delete"));
	_action_save->setText(tr("Save"));
	_action_save_as->setText(tr("Save as"));
	_action_close->setText(tr("Close"));
	_action_close_others->setText(tr("Close others"));
}

void PlaylistTabMenu::show_menu_items(bool save_action, bool save_as_action, bool delete_action, bool reset_action){
	_action_delete->setVisible(delete_action);
	_action_save_as->setVisible(save_as_action);
	_action_save->setVisible(save_action);
	_action_reset->setVisible(reset_action);
}


void PlaylistTabMenu::show_close(bool b){
	_action_close->setVisible(b);
	_action_close_others->setVisible(b);
}

