/* Helper.h */

/* Copyright (C) 2011-2016 Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Helper.cpp
 *
 *  Created on: Apr 4, 2011
 *      Author: luke
 */

#ifndef _HELPER_H
#define _HELPER_H

#include "Helper/globals.h"
#include "Helper/MetaData/MetaData.h"
#include "Helper/Logger/Logger.h"
#include "Helper/Random/RandomGenerator.h"
#include "GUI/Helper/GlobalMessage/GlobalMessage.h"

#include <QWidget>
#include <QImage>
#include <QPixmap>
#include <QIcon>
#include <QThread>
#include <QDir>
#include <QUrl>
#include <QCryptographicHash>
#include <QFile>
#include <QFontMetrics>
#include <QFileInfo>
#include <QMap>


namespace Helper{

	class StringDummy : public QObject
	{
		Q_OBJECT

	public:
		StringDummy(QObject* parent=nullptr) :
			QObject(parent)
		{

		}

		QString various();
		QString days();
		QString hours();
		QString minutes();
		QString seconds();
		QString disc();
		QString discs();
	};


	QString cvt_ms_to_string(const QObject* parent, quint64 msec, bool empty_zero = false, bool colon=true, bool show_days=true);
	QString cvt_str_to_first_upper(const QString& str);
	QString calc_filesize_str(quint64 filesize);

	QIcon get_icon(const QString& icon_name);
	QPixmap get_pixmap(const QString& icon_name, QSize sz=QSize(0, 0), bool keep_aspect=true);
	QString get_error_file();
	QString get_sayonara_path();
	QString get_share_path();
	QString get_lib_path();
	QString create_link(const QString& name,
					   const QString& target="",
					   bool underline=true);

	QString calc_cover_token(const QString& artist, const QString& album);

	QString calc_google_image_search_address(const QString& searchstring);
	QString calc_google_artist_address(const QString& artist);
	QString calc_google_album_address(const QString& artist, const QString& album);

	QString calc_lfm_artist_address(const QString& artist);
	QString calc_lfm_album_address(const QString& artist, const QString& album);

	QStringList get_soundfile_extensions();
	QStringList get_playlistfile_extensions();
    QStringList get_podcast_extensions();

	QString calc_file_extension(const QString& filename);
	void remove_files_in_directory(const QString& dir_name, const QStringList& filters);
	QString get_parent_folder(const QString& path);
	QString get_filename_of_path(const QString& path);
	void split_filename(const QString& src, QString& path, QString& filename);
	QStringList extract_folders_of_files(const QStringList& list);
	QString get_absolute_filename(const QString& filename);

	QString elide_text(const QString& text, const QWidget* widget, int max_lines);


    // Everything clear
	bool is_url(const QString& str);
	bool is_www(const QString& str);
	bool is_file(const QString& filename);
	bool is_dir(const QString& filename);
	bool is_soundfile(const QString& filename);
	bool is_playlistfile(const QString& filename);
	bool is_podcastfile(const QString& filename, const QByteArray& content);

	int get_random_number(int min, int max);



    // get album name with discnumber (calls database)
    QString get_album_w_disc(const MetaData& md);


	template<typename T>
	QList<T> randomize_list(const QList<T>& list) {

		RandomGenerator generator;
		QList<T> ret = list;
		for(auto it=ret.begin(); it!=ret.end(); it++) {
			int rnd_number = generator.get_number(0, ret.size()-1);
			std::swap(*it, *(ret.begin() + rnd_number));
		}

		return ret;
	}




	/* Checks if track exists
	 * if track does not exist
	 * it is deleted from database
	 * For online tracks it always returns true
	 * ATTENTION:
	 * Wrap this method, because for sure library has to be notified
	 */
	bool check_track(const MetaData& md);

	bool read_file_into_str(const QString& filename, QString& content);
	bool read_file_into_byte_arr(const QString& filename, QByteArray& content);

	QString easy_tag_finder(const QString&  tag, const QString& xml_doc);
	QByteArray calc_hash(const QByteArray&  data);
    QString get_album_major_artist(int album_id);
	QString get_major_artist(const QStringList& artists);
	QString get_major_artist(const ArtistList& artists);

	void sleep_ms(quint64 ms);
}

#endif
