/* AsyncWebAccess.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Helper/WebAccess/AsyncWebAccess.h"
#include "Helper/Logger/Logger.h"


AsyncWebAccess::AsyncWebAccess(QObject* parent, const QByteArray& header) :
	QObject(parent)
{
	Q_UNUSED(header)
	_nam = new QNetworkAccessManager(this);
	_timer = new QTimer();

	connect(_timer, &QTimer::timeout, this, &AsyncWebAccess::timeout);
	connect(_nam, &QNetworkAccessManager::finished, this, &AsyncWebAccess::finished);
}

AsyncWebAccess::~AsyncWebAccess() {
}


void AsyncWebAccess::set_raw_header(const QMap<QByteArray, QByteArray>& header){

	_header = header;
}

void AsyncWebAccess::run(const QString& url, int timeout){

	_url = url;
	_nam->clearAccessCache();

	QNetworkRequest request;
	request.setUrl(url);

	_reply = _nam->get(request);
	_timer->start(timeout);
}

void AsyncWebAccess::run_post(const QString &url, const QByteArray &post_data, int timeout){
	_url = url;
	_nam->clearAccessCache();

	QUrl my_url(url);
	QNetworkRequest request(my_url);

	if(!_header.isEmpty()){
		for(const QByteArray& key : _header.keys()){
			request.setRawHeader(key, _header[key]);

		}
	}

	_reply = _nam->post(request, post_data);
	_timer->start(timeout);
}


void AsyncWebAccess::finished(QNetworkReply *reply){

	bool success = (reply->error() == QNetworkReply::NoError);

	if(reply->bytesAvailable() == 0){
		QVariant var = reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
		QString redirect_url = var.toString();

		if(!redirect_url.isEmpty()){
			run(redirect_url);
			return;
		}
	}

	if(success){
		_data = reply->readAll();
	}

	else {
		_data = reply->readAll();
		sp_log(Log::Warning) << reply->errorString();
	}

	emit sig_finished(success);
}

void AsyncWebAccess::timeout(){
	_timer->stop();
	_reply->abort();
}


QByteArray AsyncWebAccess::get_data() const
{
    return _data;
}

QImage AsyncWebAccess::get_image() const
{
	QImage img;
	img.loadFromData(_data);
	return img;
}

QString AsyncWebAccess::get_url() const
{
	return _url;
}


